
#!/bin/bash
# Licensed Materials - Property of IBM
# 5724-I68
#
# (C) Copyright IBM Corp. 2000, 2006
#
# US Government Users Restricted Rights - Use, duplication, or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
# # Copyright (c) 2000, 2004 IBM Corp. - All Rights Reserved.
# # IBM makes no representations or warranties about the suitability of
# # this program, either express or implied, including but not limited to
# # the implied warranties of merchantability, fitness for a particular
# # purpose, or non-infringement. 
# 
#


# The purpose of this script is to migrate the database schema used by WPC from any FP of 5.3.2 to 5.3.2 Current FP installed
# Please consult WPC support before attempting to use this script



. ${TOP}/src/db/schema/util/set_db_localized.sh
_ERROR_LOGS=$TOP/logs/errfile.log

EXISTS=1;
NOTEXISTS=0;
count=0;

if [ -d $TOP/tmp ]; then
  TMPDIREXISTS=$EXISTS
else
  TMPDIREXISTS=$NOTEXISTS
fi

getErrorCountInLogFile()
{
    if [ ! -f $_ERROR_LOGS ]; then
        errorCount=0
    else
        errorCount=`cat $_ERROR_LOGS | wc -c`
    fi
}

DoStartAndPrepare()
{
    ${ECHO} "-----------------------------------------" 
    ${ECHO} $1
    ${ECHO} "-----------------------------------------" 

    getErrorCountInLogFile
    initial=$errorCount
}

DoEndAndReportError()
{

    getErrorCountInLogFile
    final=$errorCount

    if [ ! $initial -eq $final ]; then

        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} $1
        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} "MIGRATION_INFO_CHECK_ERROR_LOG_FILE_SH" $_ERROR_LOGS
        
    fi
    	${ECHO} ""
        ${ECHO} "-----------------------------------------"
        ${ECHO} $2
        ${ECHO} "-----------------------------------------"

}


exportACGRoles() 
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH ACG...1/2"
		for CMP_CODE_COL in $COMPANIES; 
		do
			CMP_CODE=`${ECHO} ${CMP_CODE_COL} | ${GREP} 'cmpcode%' | ${CUT} -f2 -d "%"`         
	        if [ "$CMP_CODE" != "" ]; then        	
				${ECHO} $CMP_CODE
				${ECHO} 
				
				$JAVA_RT com.ibm.ccd.envexpimp.common.EnvironmentExporter -companycode="$CMP_CODE" -scriptpath=$TOP/bin/exportACGRolesScript.wpcs 2>> $_ERROR_LOGS	
									
				$JAVA_RT com.ibm.ccd.docstore.util.DocDownloader -companycode="$CMP_CODE" -subtree=archives/fpmigration/ -xmlfile=$TOP/tmp/abc.xml -rootlocalpath=$TOP/tmp/ 2>> $_ERROR_LOGS | ${GREP} -v '.*'
				
				`mv $TOP/tmp/__archives__fpmigration__Export.zip $TOP/tmp/__archives__fpmigration__cmp"$count".zip `
			      	
				count=`expr $count + 1`
					
				`rm -f $TOP/tmp/abc.xml `
			fi            
		done
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH ACG...1/2" "MIGRATION_INFO_SUCCEEDED_SH ACG...1/2"
}


alterCategoryItemAttributes() 
{
	if [ ${CCD_DB} = "oracle" ]; then
		DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH Category Item Attributes"		
			. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/alter_occurrence_id-oracle.sql 2>> $_ERROR_LOGS			
		DoEndAndReportError  "MIGRATION_INFO_FAILED_SH Category Item Attributes" "MIGRATION_INFO_SUCCEEDED_SH Category Item Attributes"
		${ECHO} "describe ita;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "describe cab;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	fi	
}


addDateColumns()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH Profiles"     	
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/add_date_column.sql 2>> $_ERROR_LOGS
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH Profiles" "MIGRATION_INFO_SUCCEEDED_SH Profiles"
	if [ ${CCD_DB} = "oracle" ]; then
		${ECHO} "describe ppi;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "describe ppr;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "describe psi;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "describe psd;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	else
		${ECHO} "select COLNAME from syscat.columns where TABNAME = 'TPFM_PPI_PROFILE_INFO' AND TABSCHEMA = CURRENT SCHEMA;" > ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select COLNAME from syscat.columns where TABNAME = 'TPFM_PPR_PROFILE' AND TABSCHEMA = CURRENT SCHEMA;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select COLNAME from syscat.columns where TABNAME = 'TPFM_PSI_PROFILE_SCHEDULE_INFO' AND TABSCHEMA = CURRENT SCHEMA;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select COLNAME from syscat.columns where TABNAME = 'TPFM_PSD_SCHEDULE_DETAIL' AND TABSCHEMA = CURRENT SCHEMA;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	fi
}


recreateObjectActions()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH Object Actions"    	 
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_CMN_SCRIPTS/recreateOAC_OBT.sql --errorfilter=yes 2>> $_ERROR_LOGS | ${GREP} -v '.*'
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH Object Actions" "MIGRATION_INFO_SUCCEEDED_SH Object Actions"	
	${ECHO} "select count(*) NO_OF_OAC_RECORDS from oac;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	${ECHO} "select count(*) NO_OF_OBT_RECORDS from obt;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	
}


updateSequences()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH Sequences"     	
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/update_seq_table.sql 2>> $_ERROR_LOGS	
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH Sequences" "MIGRATION_INFO_SUCCEEDED_SH Sequences"
}


updateLCKIndex()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH LCK_Index"     	
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/tutl_lck_lock_indexes.sql 2>> $_ERROR_LOGS
		if [ ${CCD_DB} = "oracle" ]; then
			${ECHO} "column column_name format a30;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
			${ECHO} "select index_name, column_name, column_position from user_ind_columns where table_name= 'TUTL_LCK_LOCK' order by 1, 3;" >> ${DB_SCRIPTS}/verifyFPMigration.sql			
		else
			${ECHO} "select indname, colname, colseq from syscat.indexcoluse where indschema= CURRENT SCHEMA and indname like '%LCK%'order by indname, colseq;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		fi
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH LCK_Index" "MIGRATION_INFO_SUCCEEDED_SH LCK_Index"
}


updateWFE_NOMIndex()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH WFE_NOM_Index"
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/create_wfe_index.sql 2>> $_ERROR_LOGS
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/tctg_nom_node_mapping_index.sql 2>> $_ERROR_LOGS
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH WFE_NOM_Index" "MIGRATION_INFO_SUCCEEDED_SH WFE_NOM_Index"
	if [ ${CCD_DB} = "oracle" ]; then
		${ECHO} "column column_name format a30;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select index_name, column_name, column_position from user_ind_columns where table_name= 'TWFL_WFE_WORKFLOW_EVENT' order by 1, 3;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select index_name, column_name, column_position from user_ind_columns where table_name= 'TCTG_NOM_NODE_MAPPING' order by 1, 3;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	else
		${ECHO} "select indname, colname, colseq from syscat.indexcoluse where indschema= CURRENT SCHEMA and indname like '%WFE%'order by indname, colseq;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
		${ECHO} "select indname, colname, colseq from syscat.indexcoluse where indschema= CURRENT SCHEMA and indname like '%NOM%'order by indname, colseq;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	fi
}


createIndexExplainObject()
{
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH ExplainObject_Index"
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/create_index_explain_object.sql 2>> $_ERROR_LOGS
		${ECHO} "select indname, colname, colseq from syscat.indexcoluse where indschema= CURRENT SCHEMA and indname like '%EXP_OBJ_IND%' order by indname, colseq;" >> ${DB_SCRIPTS}/verifyFPMigration.sql
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH ExplainObject_Index" "MIGRATION_INFO_SUCCEEDED_SH ExplainObject_Index"
}


importACGRoles()
{
	count=0;
	DoStartAndPrepare "MIGRATION_INFO_UPDATING_SH ACG...2/2"
		for CMP_CODE_COL in $COMPANIES; 
			do
	            CMP_CODE=`${ECHO} ${CMP_CODE_COL} | ${GREP} 'cmpcode%' | ${CUT} -f2 -d "%"`         
	         	if [ "$CMP_CODE" != "" ]; then
	            	${ECHO} $CMP_CODE
	            	${ECHO}
	            	$JAVA_RT com.ibm.ccd.envexpimp.common.EnvironmentImporter -companycode="$CMP_CODE" -zipfilepath=$TOP/tmp/__archives__fpmigration__cmp"$count".zip 2>> $_ERROR_LOGS | ${GREP} -v '.*'
	            	            	
			      	if [ $TMPDIREXISTS -eq $EXISTS ]; then
			     		`rm $TOP/tmp/__archives__fpmigration__cmp"$count".zip `
			   		fi		
			   		
			   		count=`expr $count + 1`
	            fi            
	    done
	DoEndAndReportError  "MIGRATION_INFO_FAILED_SH ACG...2/2" "MIGRATION_INFO_SUCCEEDED_SH ACG...2/2"
	
	if [ $TMPDIREXISTS -eq $NOTEXISTS ]; then
   		`rm -rf $TOP/tmp/ `
	fi
}


GET_COMPANIES=`. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_CMN_SCRIPTS/get_cmpcode.sql`
COMPANIES="${GET_COMPANIES}"
`rm -f $DB_SCRIPTS/verifyFPMigration.sql `


if [ "${FROMVERSION}" == "BASE" ] || [ "${FROMVERSION}" == "IF1" ] || [ "${FROMVERSION}" == "IF2" ] || [ "${FROMVERSION}" == "FP1" ] || [ "${FROMVERSION}" == "FP2" ] || [ "${FROMVERSION}" == "FP3" ] || [ "${FROMVERSION}" == "IF3" ] || [ "${FROMVERSION}" == "IF4" ] || [ "${FROMVERSION}" == "IF5" ] || [ "${FROMVERSION}" == "FP4" ] || [ "${FROMVERSION}" == "FP5" ] || [ "${FROMVERSION}" == "FP6" ] || [ "${FROMVERSION}" == "IF6" ] || [ "${FROMVERSION}" == "FP7" ]; then	
	exportACGRoles
	if  [ "${FROMVERSION}" == "BASE" ] || [ "${FROMVERSION}" == "IF1" ] || [ "${FROMVERSION}" == "IF2" ]; then
		addDateColumns
		alterCategoryItemAttributes
	fi	
	if  [ "${FROMVERSION}" == "BASE" ]  || [ "${FROMVERSION}" == "IF1" ] || [ "${FROMVERSION}" == "IF2" ] || [ "${FROMVERSION}" == "FP1" ]; then
		updateSequences
	fi	
	recreateObjectActions
	if  [ "${FROMVERSION}" != "FP6" ]  && [ "${FROMVERSION}" != "IF6" ] && [ "${FROMVERSION}" != "FP7" ]; then
		updateLCKIndex
	fi		
	updateWFE_NOMIndex
	if [ ${CCD_DB} = "db2" ]; then
		createIndexExplainObject
		updateLCKIndex
	fi	
	#*******************************************************************
	#  Functions for schema changes done in FP10 and above
	#  would be included here
	#
	#*******************************************************************
	importACGRoles
elif [ "${FROMVERSION}" == "IF7" ]; then
	updateWFE_NOMIndex
	if [ ${CCD_DB} = "db2" ]; then
		createIndexExplainObject
		updateLCKIndex
	fi
	#*******************************************************************
	#  Functions for schema changes done in FP10 and above
	#  would be included here
	#
	#*******************************************************************
	
elif [ "${FROMVERSION}" == "FP8" ]; then
	if [ ${CCD_DB} = "db2" ]; then		
		createIndexExplainObject
		updateLCKIndex
	else
		exit -1
	fi
	#*******************************************************************
	#  Functions for schema changes done in FP10 and above
	#  would be included here
	#
	#*******************************************************************	
fi



#Verification step
`rm -f $TOP/logs/verify.log ` 
. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/verifyFPMigration.sql --errorfilter=yes >> $TOP/logs/verify.log




